@@ -1,5 +1,5 @@
 /* Handles parsing the Options provided to the user.
-   Copyright (C) 1989-1998, 2000, 2002-2004, 2006-2009 Free Software Foundation, Inc.
+   Copyright (C) 1989-1998, 2000, 2002-2004, 2006-2009, 2011, 2016-2018, 2022-2023, 2025 Free Software Foundation, Inc.
    Written by Douglas C. Schmidt <schmidt@ics.uci.edu>
    and Bruno Haible <bruno@clisp.org>.
 
@@ -16,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Specification. */
 #include "options.h"
@@ -26,6 +26,7 @@
 #include <string.h> /* declares strcmp() */
 #include <ctype.h>  /* declares isdigit() */
 #include <limits.h> /* defines CHAR_MAX */
+#include "filename.h"
 #include "getopt.h"
 #include "version.h"
 
@@ -62,6 +63,9 @@ static const char *const DEFAULT_LENGTHTABLE_NAME = "lengthtable";
 /* Default name for string pool.  */
 static const char *const DEFAULT_STRINGPOOL_NAME = "stringpool";
 
+/* Default prefix for constants.  */
+static const char *const DEFAULT_CONSTANTS_PREFIX = "";
+
 /* Default delimiters that separate keywords from their attributes.  */
 static const char *const DEFAULT_DELIMITERS = ",";
 
@@ -120,7 +124,7 @@ Options::long_usage (FILE * stream)
            "  -L, --language=LANGUAGE-NAME\n"
            "                         Generates code in the specified language. Languages\n"
            "                         handled are currently C++, ANSI-C, C, and KR-C. The\n"
-           "                         default is C.\n");
+           "                         default is ANSI-C.\n");
   fprintf (stream, "\n");
   fprintf (stream,
            "Details in the output code:\n");
@@ -176,6 +180,9 @@ Options::long_usage (FILE * stream)
   fprintf (stream,
            "      --null-strings     Use NULL strings instead of empty strings for empty\n"
            "                         keyword table entries.\n");
+  fprintf (stream,
+           "      --constants-prefix=PREFIX\n"
+           "                         Specify prefix for the constants like TOTAL_KEYWORDS.\n");
   fprintf (stream,
            "  -W, --word-array-name=NAME\n"
            "                         Specify name of word list array. Default name is\n"
@@ -260,7 +267,7 @@ Options::long_usage (FILE * stream)
            "                         output to the standard error).\n");
   fprintf (stream, "\n");
   fprintf (stream,
-           "Report bugs to <bug-gnu-gperf@gnu.org>.\n");
+           "Report bugs to <bug-gperf@gnu.org>.\n");
 }
 
 /* Prints the given options.  */
@@ -272,14 +279,36 @@ Options::print_options () const
 
   for (int i = 0; i < _argument_count; i++)
     {
-      const char *arg = _argument_vector[i];
+      char *arg = _argument_vector[i];
+
+      if (i == 0)
+        {
+          /* _argument_vector[0] is the program name.  Print only its base name,
+             and also drop its ".exe" suffix on native Windows.
+             This is useful for reproducible builds.  */
+          {
+            char *p = arg + strlen (arg);
+            while (p > arg && ! ISSLASH (p[-1]))
+              p--;
+            arg = p;
+          }
+          {
+            char *p = arg + strlen (arg);
+            if (p - arg > 4
+                && p[-4] == '.'
+                && (p[-3] == 'E' || p[-3] == 'e')
+                && (p[-2] == 'X' || p[-2] == 'x')
+                && (p[-1] == 'E' || p[-1] == 'e'))
+              p[-4] = '\0';
+          }
+        }
 
       /* Escape arg if it contains shell metacharacters.  */
       if (*arg == '-')
         {
           putchar (*arg);
           arg++;
-          if (*arg >= 'A' && *arg <= 'Z' || *arg >= 'a' && *arg <= 'z')
+          if ((*arg >= 'A' && *arg <= 'Z') || (*arg >= 'a' && *arg <= 'z'))
             {
               putchar (*arg);
               arg++;
@@ -291,7 +320,7 @@ Options::print_options () const
                   putchar (*arg);
                   arg++;
                 }
-              while (*arg >= 'A' && *arg <= 'Z' || *arg >= 'a' && *arg <= 'z' || *arg == '-');
+              while ((*arg >= 'A' && *arg <= 'Z') || (*arg >= 'a' && *arg <= 'z') || *arg == '-');
               if (*arg == '=')
                 {
                   putchar (*arg);
@@ -448,7 +477,7 @@ PositionStringParser::nextPosition ()
 /* Sets the default Options.  */
 
 Options::Options ()
-  : _option_word (C),
+  : _option_word (ANSIC),
     _input_file_name (NULL),
     _output_file_name (NULL),
     _language (NULL),
@@ -465,6 +494,7 @@ Options::Options ()
     _wordlist_name (DEFAULT_WORDLIST_NAME),
     _lengthtable_name (DEFAULT_LENGTHTABLE_NAME),
     _stringpool_name (DEFAULT_STRINGPOOL_NAME),
+    _constants_prefix (DEFAULT_CONSTANTS_PREFIX),
     _delimiters (DEFAULT_DELIMITERS),
     _key_positions ()
 {
@@ -574,9 +604,10 @@ Options::set_language (const char *language)
         _option_word |= CPLUSPLUS;
       else
         {
-          fprintf (stderr, "unsupported language option %s, defaulting to C\n",
+          fprintf (stderr,
+                   "unsupported language option %s, defaulting to ANSI-C\n",
                    language);
-          _option_word |= C;
+          _option_word |= ANSIC;
         }
     }
 }
@@ -608,6 +639,13 @@ Options::set_slot_name (const char *name)
     _slot_name = name;
 }
 
+/* Returns true if the struct initializer suffix has been set.  */
+bool
+Options::has_initializer_suffix () const
+{
+  return _initializer_suffix != DEFAULT_INITIALIZER_SUFFIX;
+}
+
 /* Sets the struct initializer suffix, if not already set.  */
 void
 Options::set_initializer_suffix (const char *initializers)
@@ -648,6 +686,14 @@ Options::set_lengthtable_name (const char *name)
     _lengthtable_name = name;
 }
 
+/* Sets the prefix for the constants, if not already set.  */
+void
+Options::set_constants_prefix (const char *prefix)
+{
+  if (_constants_prefix == DEFAULT_CONSTANTS_PREFIX)
+    _constants_prefix = prefix;
+}
+
 /* Sets the string pool name, if not already set.  */
 void
 Options::set_stringpool_name (const char *name)
@@ -687,6 +733,7 @@ static const struct option long_options[] =
   { "enum", no_argument, NULL, 'E' },
   { "includes", no_argument, NULL, 'I' },
   { "global-table", no_argument, NULL, 'G' },
+  { "constants-prefix", required_argument, NULL, CHAR_MAX + 5 },
   { "word-array-name", required_argument, NULL, 'W' },
   { "length-table-name", required_argument, NULL, CHAR_MAX + 4 },
   { "switch", required_argument, NULL, 'S' },
@@ -1000,11 +1047,11 @@ Options::parse_options (int argc, char *argv[])
         case 'v':               /* Print out the version and quit.  */
           fprintf (stdout, "GNU gperf %s\n", version_string);
           fprintf (stdout, "Copyright (C) %s Free Software Foundation, Inc.\n\
-License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n\
+License GPLv3+: GNU GPL version 3 or later <https://gnu.org/licenses/gpl.html>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n\
 ",
-                   "1989-1998, 2000-2004, 2006-2009");
+                   "2025");
           fprintf (stdout, "Written by %s and %s.\n",
                    "Douglas C. Schmidt", "Bruno Haible");
           exit (0);
@@ -1043,6 +1090,11 @@ There is NO WARRANTY, to the extent permitted by law.\n\
             _lengthtable_name = /*getopt*/optarg;
             break;
           }
+        case CHAR_MAX + 5:      /* Sets the prefix for the constants.  */
+          {
+            _constants_prefix = /*getopt*/optarg;
+            break;
+          }
         default:
           short_usage (stderr);
           exit (1);
