@@ -1,5 +1,5 @@
 /* Output routines.
-   Copyright (C) 1989-1998, 2000, 2002-2004, 2006-2007, 2009, 2011-2012, 2016 Free Software Foundation, Inc.
+   Copyright (C) 1989-2025 Free Software Foundation, Inc.
    Written by Douglas C. Schmidt <schmidt@ics.uci.edu>
    and Bruno Haible <bruno@clisp.org>.
 
@@ -16,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Specification. */
 #include "output.h"
@@ -288,8 +288,9 @@ output_upperlower_table ()
 
   printf ("#ifndef GPERF_DOWNCASE\n"
           "#define GPERF_DOWNCASE 1\n"
-          "static unsigned char gperf_downcase[256] =\n"
-          "  {");
+          "static %sunsigned char gperf_downcase[256] =\n"
+          "  {",
+          const_readonly_array);
   for (c = 0; c < 256; c++)
     {
       if ((c % 15) == 0)
@@ -814,12 +815,15 @@ Output::output_hash_function () const
             "#endif\n"
             "#endif\n");
 
-  if (/* The function does not use the 'str' argument?  */
-      _key_positions.get_size() == 0
-      || /* The function uses 'str', but not the 'len' argument?  */
-         (!_hash_includes_len
-          && _key_positions[0] < _min_key_len
-          && _key_positions[_key_positions.get_size() - 1] != Positions::LASTCHAR))
+  /* Does the function use the 'str' argument?  */
+  bool uses_str = (_key_positions.get_size() > 0);
+  /* Does the function use the 'len' argument?  */
+  bool uses_len =
+     (_hash_includes_len
+      || (_key_positions.get_size() > 0
+          && (_key_positions[0] >= _min_key_len
+              || _key_positions[_key_positions.get_size() - 1] == Positions::LASTCHAR)));
+  if (!uses_str || !uses_len)
     /* Pacify lint.  */
     printf ("/*ARGSUSED*/\n");
 
@@ -829,6 +833,9 @@ Output::output_hash_function () const
   if (option[CPLUSPLUS])
     printf ("%s::", option.get_class_name ());
   printf ("%s ", option.get_hash_name ());
+  /* We better not use [[__maybe_unused__]] or __attribute__ ((__unused__))
+     because support for these syntaxes in the compilers is constantly
+     changing.  */
   printf (option[KRC] ?
                  "(str, len)\n"
             "     %schar *str;\n"
@@ -851,6 +858,8 @@ Output::output_hash_function () const
   /* First the asso_values array.  */
   if (_key_positions.get_size() > 0)
     {
+      /* The values in the asso_values array are all unsigned integers
+         <= MAX_HASH_VALUE + 1.  */
       printf ("  static %s%s asso_values[] =\n"
               "    {",
               const_readonly_array,
@@ -858,9 +867,9 @@ Output::output_hash_function () const
 
       const int columns = 10;
 
-      /* Calculate maximum number of digits required for MAX_HASH_VALUE.  */
+      /* Calculate maximum number of digits required for MAX_HASH_VALUE + 1.  */
       int field_width = 2;
-      for (int trunc = _max_hash_value; (trunc /= 10) > 0;)
+      for (int trunc = _max_hash_value + 1; (trunc /= 10) > 0;)
         field_width++;
 
       for (unsigned int count = 0; count < _alpha_size; count++)
@@ -876,6 +885,15 @@ Output::output_hash_function () const
               "    };\n");
     }
 
+  if (!uses_str)
+    /* The function does not use the 'str' argument.
+       Silence "gcc -Wunused-parameter".  */
+    printf ("  (void) str;\n");
+  if (!uses_len)
+    /* The function does not use the 'len' argument.
+       Silence "gcc -Wunused-parameter".  */
+    printf ("  (void) len;\n");
+
   if (_key_positions.get_size() == 0)
     {
       /* Trivial case: No key positions at all.  */
@@ -932,6 +950,15 @@ Output::output_hash_function () const
       else
         {
           /* We've got to use the correct, but brute force, technique.  */
+          /* Pseudo-statement or comment that avoids a compiler warning or
+             lint warning.  */
+          const char * const fallthrough_marker =
+            "#if (defined __cplusplus && (__cplusplus >= 201703L || (__cplusplus >= 201103L && defined __clang__ && __clang_major__ + (__clang_minor__ >= 9) > 3))) || (defined __STDC_VERSION__ && __STDC_VERSION__ >= 202000L && ((defined __GNUC__ && __GNUC__ >= 10) || (defined __clang__ && __clang_major__ >= 9)))\n"
+            "      [[fallthrough]];\n"
+            "#elif (defined __GNUC__ && __GNUC__ >= 7) || (defined __clang__ && __clang_major__ >= 10)\n"
+            "      __attribute__ ((__fallthrough__));\n"
+            "#endif\n"
+            "      /*FALLTHROUGH*/\n";
           /* It doesn't really matter whether hval is an 'int' or
              'unsigned int', but 'unsigned int' gives fewer warnings.  */
           printf ("  %sunsigned int hval = %s;\n\n"
@@ -951,7 +978,7 @@ Output::output_hash_function () const
               do
                 {
                   if (i > key_pos)
-                    printf ("      /*FALLTHROUGH*/\n"); /* Pacify lint.  */
+                    printf ("%s", fallthrough_marker);
                   for ( ; i > key_pos; i--)
                     printf ("      case %d:\n", i);
 
@@ -964,7 +991,7 @@ Output::output_hash_function () const
               while (key_pos != PositionIterator::EOS && key_pos != Positions::LASTCHAR);
 
               if (i >= _min_key_len)
-                printf ("      /*FALLTHROUGH*/\n"); /* Pacify lint.  */
+                printf ("%s", fallthrough_marker);
               for ( ; i >= _min_key_len; i--)
                 printf ("      case %d:\n", i);
             }
@@ -1161,7 +1188,7 @@ Output::output_string_pool () const
 /* ------------------------------------------------------------------------- */
 
 static void
-output_keyword_entry (KeywordExt *temp, int stringpool_index, const char *indent)
+output_keyword_entry (KeywordExt *temp, int stringpool_index, const char *indent, bool is_duplicate)
 {
   if (option[TYPE])
     output_line_directive (temp->_lineno);
@@ -1194,8 +1221,14 @@ output_keyword_entry (KeywordExt *temp, int stringpool_index, const char *indent
       printf ("}");
     }
   if (option[DEBUG])
-    printf (" /* hash value = %d, index = %d */",
-            temp->_hash_value, temp->_final_index);
+    {
+      printf (" /* ");
+      if (is_duplicate)
+        printf ("hash value duplicate, ");
+      else
+        printf ("hash value = %d, ", temp->_hash_value);
+      printf ("index = %d */", temp->_final_index);
+    }
 }
 
 static void
@@ -1249,10 +1282,24 @@ output_keyword_blank_entries (int count, const char *indent)
 void
 Output::output_keyword_table () const
 {
-  const char *indent  = option[GLOBAL] ? "" : "  ";
+  const char *indent = option[GLOBAL] ? "" : "  ";
   int index;
   KeywordExt_List *temp;
 
+  /* Avoid compiler warnings "warning: missing initializer for field ..."
+     for each of the blank entries.  */
+  bool silence_missing_initializer_warning =
+    option[TYPE] && !option.has_initializer_suffix();
+  const char *preprocessor_condition =
+    "(defined __GNUC__ && __GNUC__ + (__GNUC_MINOR__ >= 6) > 4) || (defined __clang__ && __clang_major__ >= 3)";
+  if (silence_missing_initializer_warning)
+    {
+      printf ("#if %s\n", preprocessor_condition);
+      printf ("#pragma GCC diagnostic push\n");
+      printf ("#pragma GCC diagnostic ignored \"-Wmissing-field-initializers\"\n");
+      printf ("#endif\n");
+    }
+
   printf ("%sstatic ",
           indent);
   output_const_type (const_readonly_array, _wordlist_eltype);
@@ -1286,7 +1333,7 @@ Output::output_keyword_table () const
 
       keyword->_final_index = index;
 
-      output_keyword_entry (keyword, index, indent);
+      output_keyword_entry (keyword, index, indent, false);
 
       /* Deal with duplicates specially.  */
       if (keyword->_duplicate_link) // implies option[DUP]
@@ -1300,7 +1347,7 @@ Output::output_keyword_table () const
                           keyword->_allchars_length) == 0
                ? keyword->_final_index
                : links->_final_index);
-            output_keyword_entry (links, stringpool_index, indent);
+            output_keyword_entry (links, stringpool_index, indent, true);
           }
 
       index++;
@@ -1308,7 +1355,15 @@ Output::output_keyword_table () const
   if (index > 0)
     printf ("\n");
 
-  printf ("%s  };\n\n", indent);
+  printf ("%s  };\n", indent);
+
+  if (silence_missing_initializer_warning)
+    {
+      printf ("#if %s\n", preprocessor_condition);
+      printf ("#pragma GCC diagnostic pop\n");
+      printf ("#endif\n");
+    }
+  printf ("\n");
 }
 
 /* ------------------------------------------------------------------------- */
@@ -1652,6 +1707,17 @@ output_switches (KeywordExt_List *list, int num_switches, int size, int min_hash
 void
 Output::output_lookup_function_body (const Output_Compare& comparison) const
 {
+  /* An expression equivalent to NULL, of type _return_type.
+     We don't use plain "0", because that would trigger a gcc 15 warning
+     if the warning option -Wzero-as-null-pointer-constant is in use.
+     In C++, using "nullptr" if __cplusplus >= 201103L would be possible,
+     but is not worth the trouble.  */
+  char *null_expression = new char[17 + strlen (_return_type) + 1];
+  if (option[CPLUSPLUS])
+    sprintf (null_expression, "static_cast<%s> (0)", _return_type);
+  else
+    sprintf (null_expression, "(%s) 0", _return_type);
+
   printf ("  if (len <= %sMAX_WORD_LENGTH && len >= %sMIN_WORD_LENGTH)\n"
           "    {\n"
           "      %sunsigned int key = %s (str, len);\n\n",
@@ -1700,7 +1766,8 @@ Output::output_lookup_function_body (const Output_Compare& comparison) const
 
       output_switches (_head, num_switches, switch_size, _min_hash_value, _max_hash_value, 10);
 
-      printf ("          return 0;\n");
+      printf ("          return %s;\n",
+              null_expression);
       if (option[DUP] && _total_duplicates > 0)
         {
           int indent = 8;
@@ -1743,8 +1810,8 @@ Output::output_lookup_function_body (const Output_Compare& comparison) const
                     indent, "");
           printf ("%*s      wordptr++;\n"
                   "%*s    }\n"
-                  "%*s  return 0;\n",
-                  indent, "", indent, "", indent, "");
+                  "%*s  return %s;\n",
+                  indent, "", indent, "", indent, "", null_expression);
         }
       printf ("        compare:\n");
       if (option[TYPE])
@@ -1965,7 +2032,10 @@ Output::output_lookup_function_body (const Output_Compare& comparison) const
         }
     }
   printf ("    }\n"
-          "  return 0;\n");
+          "  return %s;\n",
+          null_expression);
+
+  delete[] null_expression;
 }
 
 /* Generates C code for the lookup function.  */
