@@ -1,5 +1,5 @@
 /* Search algorithm.
-   Copyright (C) 1989-1998, 2000, 2002, 2009, 2016 Free Software Foundation, Inc.
+   Copyright (C) 1989-1998, 2000, 2002-2003, 2009, 2016 Free Software Foundation, Inc.
    Written by Douglas C. Schmidt <schmidt@ics.uci.edu>
    and Bruno Haible <bruno@clisp.org>.
 
@@ -16,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* Specification. */
 #include "search.h"
@@ -413,13 +413,15 @@ Search::find_positions ()
       Positions best;
       unsigned int best_duplicates_count = UINT_MAX;
 
+      /* Loop over all pairs { i1, i2 } of currently selected positions.
+         W.l.o.g. we can assume i1 > i2.  */
       for (int i1 = imax; i1 >= -1; i1--)
         if (current.contains (i1) && !mandatory.contains (i1))
           {
-            for (int i2 = imax; i2 >= -1; i2--)
-              if (current.contains (i2) && !mandatory.contains (i2) && i2 != i1)
+            for (int i2 = i1 - 1; i2 >= -1; i2--)
+              if (current.contains (i2) && !mandatory.contains (i2))
                 {
-                  for (int i3 = imax; i3 >= 0; i3--)
+                  for (int i3 = imax; i3 >= -1; i3--)
                     if (!current.contains (i3))
                       {
                         Positions tryal = current;
@@ -435,7 +437,7 @@ Search::find_positions ()
                            function.  */
                         if (try_duplicates_count < best_duplicates_count
                             || (try_duplicates_count == best_duplicates_count
-                                && (i1 == -1 || i2 == -1 || i3 >= 0)))
+                                && (i1 == -1 || i2 == -1) && i3 >= 0))
                           {
                             best = tryal;
                             best_duplicates_count = try_duplicates_count;
